/*

Client program for APC test driver
==================================

By Enrico Martignetti

*/

#include <windows.h>
#include <stdio.h>
#include <tchar.h>

#include <CodeAnalysis/warnings.h>

#pragma warning(push)

#pragma warning(disable: ALL_CODE_ANALYSIS_WARNINGS)

#include <strsafe.h>

#pragma warning(pop)


#include "DrvR3.h"

// Command line defines
// --------------------

// Function codes
//
#define CLTFUN_DISABLE			L'd'
#define CLTFUN_USER_MODE		L'u'
#define CLTFUN_USER_MODE2		L'2'
#define CLTFUN_SPURIOUS_INT		L's'
#define CLTFUN_HELP				L'h'



// Other defines
// -------------

#define CLT_ERROR	100

BOOLEAN CheckParams(
	int		argC, 
	wchar_t	*lpwszArgV[]);

VOID PrintHelp();

int wmain( int argC, wchar_t *lpwszArgV[], wchar_t *lpwszEnvV[]) {


	DWORD	dwBytesRet;
	wchar_t	Function;
	HANDLE	hDevice;
	int		RetCode;

	wprintf(L"\nAPC Test client, compiled %S %S\n\n",__DATE__, __TIME__);

	if (!CheckParams(argC, lpwszArgV)) {
		PrintHelp();
		return CLT_ERROR;
	}
	// Load the driver
	//
    hDevice =
        CreateFile( L"\\\\.\\"DRV_DEVICE_NAME,
					GENERIC_READ | GENERIC_WRITE,
					FILE_SHARE_READ | FILE_SHARE_WRITE,
					NULL,	// no security
					OPEN_EXISTING,
					FILE_ATTRIBUTE_NORMAL,
					NULL );		// no template

	if ( hDevice == INVALID_HANDLE_VALUE ) {
		wprintf(
			L"Failed to obtain file handle to %s. "
			L"GetLastError() = %d (0x%x)\n",
			DRV_DEVICE_NAME,
			GetLastError(),
			GetLastError());
        return CLT_ERROR;
	}
	Function = *(lpwszArgV[1] + 1);
	switch (Function) {
		case CLTFUN_HELP:
			PrintHelp();
			RetCode = 0;
			break;
		case CLTFUN_DISABLE:

			// APC disable test
			
			DeviceIoControl(
				hDevice,
				IOCTL_APCTEST_DISABLE_TEST,
				NULL,
				0,
				NULL,
				0,
				&dwBytesRet,
				NULL);
			RetCode = 0;
			break;
		case CLTFUN_USER_MODE:

			// APC user mode test
			
			DeviceIoControl(
				hDevice,
				IOCTL_APCTEST_USER_MODE_TEST,
				NULL,
				0,
				NULL,
				0,
				&dwBytesRet,
				NULL);
			RetCode = 0;
			break;
		case CLTFUN_USER_MODE2:

			// APC user mode test 2
			
			DeviceIoControl(
				hDevice,
				IOCTL_APCTEST_USER_MODE_TEST2,
				NULL,
				0,
				NULL,
				0,
				&dwBytesRet,
				NULL);
			RetCode = 0;
			break;
		case CLTFUN_SPURIOUS_INT:

			// APC spurious int test
			
			DeviceIoControl(
				hDevice,
				IOCTL_APCTEST_SPURIOUS_INT_TEST,
				NULL,
				0,
				NULL,
				0,
				&dwBytesRet,
				NULL);
			RetCode = 0;
			break;
		default:
			wprintf(L"\nInvalid command: %s", lpwszArgV[1]);
			RetCode = CLT_ERROR;
	}
	return RetCode;
}


//++
// Function:	CheckParams
//
// Description:
//
//		Check the command line params.
//
// Arguments:
//
//		argC - from the command line.
//
//		lpwszArgV - from the command line.
//
//
// Return value: TRUE if the params are correct, FALSE otherwise.
//--
BOOLEAN CheckParams(
	int		argC, 
	wchar_t	*lpwszArgV[])
{
	if (argC < 2) {
		wprintf(L"\n\nCommand missing\n");
		return FALSE;
	}
	if (wcslen(lpwszArgV[1]) != 2) {
		wprintf(L"\n\nCommand format not valid: %s\n", lpwszArgV[1]);
		return FALSE;
	}
	if (*lpwszArgV[1] != L'/') {
		wprintf(L"\n\nCommand format not valid: %s\n", lpwszArgV[1]);
		return FALSE;
	}
	switch (*(lpwszArgV[1] + 1)) {
		case CLTFUN_DISABLE:
		case CLTFUN_HELP:
		case CLTFUN_SPURIOUS_INT:
		case CLTFUN_USER_MODE:
		case CLTFUN_USER_MODE2:
			return TRUE;
			break;
		default:
			wprintf(L"\n\nInvalid command: %s\n", lpwszArgV[1]);
			return FALSE;
	}
}


//++
//--
VOID PrintHelp()
{
	wprintf(L"\n\nUsage:");
	wprintf(L"\n");
	wprintf(L"\n    TestClt /d - APC disable test");
	wprintf(L"\n    TestClt /u - APC user mode test");
	wprintf(L"\n    TestClt /2 - APC user mode test #2");
	wprintf(L"\n    TestClt /s - APC spurious int test - VISTA SP1 ONLY. OTHER VER => BSOD!!!");
	wprintf(L"\n");
	wprintf(L"\n    TestClt /h - This help");
	wprintf(L"\n");
	wprintf(L"\n");
}


